@@ -7,8 +7,8 @@
 
 assert(luaotfload_module, "This is a part of luaotfload and should not be loaded independently") { 
     name          = "luaotfload-diagnostics",
-    version       = "3.16",       --TAGVERSION
-    date          = "2020-12-31", --TAGDATE
+    version       = "3.22",       --TAGVERSION
+    date          = "2022-06-15", --TAGDATE
     description   = "luaotfload-tool submodule / diagnostics",
     license       = "GPL v2.0"
 }
@@ -376,7 +376,7 @@ else
                 out ("Try again later at %s.", osdate ("%F %T", reset))
             else --- windows doesn’t C99
                 out ("Try again later at %s.",
-                     osdate ("%Y-%m-d %H:%M:%S", reset))
+                     osdate ("%Y-%m-%d %H:%M:%S", reset))
             end
         end
         return true
