@@ -2,17 +2,15 @@
 --         FILE:  luaotfload-harf-plug.lua
 --  DESCRIPTION:  part of luaotfload / HarfBuzz / fontloader plugin
 -----------------------------------------------------------------------
-do -- block to avoid to many local variables error
- assert(luaotfload_module, "This is a part of luaotfload and should not be loaded independently") { 
-     name          = "luaotfload-harf-plug",
-     version       = "3.16",       --TAGVERSION
-     date          = "2020-12-31", --TAGDATE
-     description   = "luaotfload submodule / HarfBuzz shaping",
-     license       = "GPL v2.0",
-     author        = "Khaled Hosny, Marcel Krüger",
-     copyright     = "Luaotfload Development Team",     
- }
-end
+assert(luaotfload_module, "This is a part of luaotfload and should not be loaded independently") {
+  name          = "luaotfload-harf-plug",
+  version       = "3.22",       --TAGVERSION
+  date          = "2022-06-15", --TAGDATE
+  description   = "luaotfload submodule / HarfBuzz shaping",
+  license       = "GPL v2.0",
+  author        = "Khaled Hosny, Marcel Krüger",
+  copyright     = "Luaotfload Development Team",
+}
 
 local hb                = luaotfload.harfbuzz
 local logreport         = luaotfload.log.report
@@ -112,6 +110,11 @@ local endactual_p       = "luaotfload_endactualtext"
 
 local empty_table       = {}
 
+local function is_empty_disc(n)
+  local pre, post, rep = getdisc(n)
+  return not pre and not post and not rep
+end
+
 -- "Copy" properties as done by LuaTeX: Make old properties metatable 
 local function copytable(old)
   local new = {}
@@ -133,8 +136,8 @@ local function setprop(n, prop, value)
   props[prop] = value
 end
 
-local function inherit(t, base, properties)
-  local n = newnode(t)
+local function inherit(t, sub, base, properties)
+  local n = newnode(t, sub)
   setattrs(n, getattrs(base))
   if properties then
     setproperty(n, setmetatable({}, {__index = properties}))
@@ -143,7 +146,7 @@ local function inherit(t, base, properties)
 end
 -- New kern node of amount `v`, inheriting the properties/attributes of `n`.
 local function newkern(v, n)
-  local kern = inherit(kern_t, n, getproperty(n))
+  local kern = inherit(kern_t, 0, n, getproperty(n))
   setkern(kern, v)
   return kern
 end
@@ -176,19 +179,21 @@ local function itemize(head, fontid, direction)
   local hbdata   = fontdata and fontdata.hb
   local spec     = fontdata and fontdata.specification
   local options  = spec and spec.features.raw
+  local obj_repl = hbdata and hbdata.obj_repl
 
   local runs, codes = {}, {}
   local dirstack = {}
   local currdir = direction or 0
   local lastskip, lastdir = true
-  local lastrun = {}
+  local dummyrun = { start = 0, len = 0 }
+  local lastrun = dummyrun
   local lastdisc
   local in_disc
 
   for n, id, subtype in traverse(head) do
     if in_disc == n then in_disc = nil end
     local disc
-    local code = 0xFFFC -- OBJECT REPLACEMENT CHARACTER
+    local code = obj_repl -- OBJECT REPLACEMENT CHARACTER, substitute invalid surrogate if otherwise colliding with font glyph
     local skip = lastskip
     local props = properties[n]
 
@@ -206,7 +211,7 @@ local function itemize(head, fontid, direction)
     elseif id == glue_t and subtype == spaceskip_t then
       code = 0x0020 -- SPACE
     elseif id == disc_t then
-      if uses_font(n, fontid) then
+      if uses_font(n, fontid) or not lastskip and is_empty_disc(n) then
         local _, _, rep, _, _, rep_tail = getdisc(n, true)
         setfield(n, 'replace', nil)
         local prev, next = getboth(n)
@@ -226,9 +231,6 @@ local function itemize(head, fontid, direction)
         setlink(prev, n)
         code = nil
         skip = false
-        if not prev then
-          head = n
-        end
       else
         skip = true
       end
@@ -272,6 +274,9 @@ local function itemize(head, fontid, direction)
       lastrun.len = lastrun.len + 1
     elseif disc then
       if lastdisc then
+        if lastrun.len == 0 then
+          runs[#runs - 1].after = n
+        end
         lastdisc.next = disc
         lastdisc = disc
       else
@@ -280,7 +285,7 @@ local function itemize(head, fontid, direction)
     end
   end
 
-  return head, runs
+  return dummyrun.after, runs
 end
 
 
@@ -288,7 +293,7 @@ end
 local function unsafetobreak(glyph)
   return glyph
      and glyph.flags
-     and glyph.flags & fl_unsafe
+     and glyph.flags & fl_unsafe ~= 0
 end
 
 local shape
@@ -348,6 +353,7 @@ function shape(head, firstnode, run)
   local options = spec.features.raw
   local hbshared = hbdata.shared
   local hbfont = hbshared.font
+  local obj_repl = hbdata.obj_repl
 
   local lang = spec.language
   local script = spec.script
@@ -431,7 +437,9 @@ function shape(head, firstnode, run)
       end
 
         -- Is this a safe breakpoint?
-      if discs and ((not glyph) or codes[cluster] == 0x20 or codes[cluster+1] == 0x20 or codes[cluster+1] == 0xFFFC
+      if discs and ((not glyph)
+            or codes[cluster] == 0x20 or codes[cluster+1] == 0x20
+            or codes[cluster] == obj_repl or codes[cluster+1] == obj_repl
           or not unsafetobreak(glyph)) then
         -- Should we change the discretionary state?
         local anchor_cluster, after_cluster = offset + discs.anchor_cluster, offset + discs.after_cluster
@@ -439,6 +447,7 @@ function shape(head, firstnode, run)
         while disc_cluster and after_cluster <= cluster
            or not disc_cluster and anchor_cluster <= cluster do
           if disc_cluster then
+            if false then -- The saved_... are for nested (aka first/second or init/select) discretionary nodes. The code is currently disabled because the linebreaking for these is broken on the engine level.
             if not saved_after and saved_anchor < cluster then
               saved_after = discs.next.after_cluster + offset
               if saved_after > cluster then
@@ -448,11 +457,14 @@ function shape(head, firstnode, run)
             elseif saved_after then
               saved_after, after_cluster = after_cluster, saved_after
             end
+            end
             local rep_glyphs = table.move(glyphs, disc_glyph, i - 1, 1, {})
             for j = 1, #rep_glyphs do
               local glyph = rep_glyphs[j]
               glyph.cluster = glyph.cluster - disc_cluster
-              glyph.nextcluster = glyph.nextcluster - disc_cluster
+              if glyph.nextcluster then
+                glyph.nextcluster = glyph.nextcluster - disc_cluster
+              end
             end
             do
               local cluster_offset = disc_cluster - cluster + (saved_after and 2 or 1) -- The offset the glyph indices will move
@@ -475,20 +487,20 @@ function shape(head, firstnode, run)
             local precodes, postcodes, repcodes = {}, {}
             table.move(codes, disc_cluster + 1, anchor_cluster, 1, precodes)
             for n in traverse(pre) do
-              precodes[#precodes + 1] = is_char(n, fontid) or 0xFFFC
+              precodes[#precodes + 1] = is_char(n, fontid) or obj_repl
             end
             for n in traverse(post) do
-              postcodes[#postcodes + 1] = is_char(n, fontid) or 0xFFFC
+              postcodes[#postcodes + 1] = is_char(n, fontid) or obj_repl
             end
             table.move(codes, after_cluster + 1, cluster, #postcodes + 1, postcodes)
 
             if saved_after then
               repcodes = table.move(codes, disc_cluster + 1, cluster, 1, {})
               table.move(codes, cluster + 1, #codes + cluster - disc_cluster, disc_cluster + 3)
-              codes[disc_cluster + 1], codes[disc_cluster + 2] = 0xFFFC, 0xFFFC
+              codes[disc_cluster + 1], codes[disc_cluster + 2] = obj_repl, obj_repl
             else
               table.move(codes, cluster + 1, #codes + cluster - disc_cluster, disc_cluster + 2)
-              codes[disc_cluster + 1] = 0xFFFC
+              codes[disc_cluster + 1] = obj_repl
             end
 
             do
@@ -532,7 +544,7 @@ function shape(head, firstnode, run)
               post = makesub(run, postcodes, post),
               cluster = disc_cluster,
               nextcluster = disc_cluster + 1,
-              codepoint = 0xFFFC,
+              codepoint = obj_repl,
             }
             if saved_after then
               local next_disc = discs.next
@@ -547,10 +559,10 @@ function shape(head, firstnode, run)
                 local saved_anchor, saved_after = saved_anchor + saved_offset, saved_after + saved_offset
                 table.move(postcodes, 1, saved_anchor, 1, next_precodes)
                 for n in traverse(next_pre) do
-                  next_precodes[#next_precodes + 1] = is_char(n, fontid) or 0xFFFC
+                  next_precodes[#next_precodes + 1] = is_char(n, fontid) or obj_repl
                 end
                 for n in traverse(next_post) do
-                  next_postcodes[#next_postcodes + 1] = is_char(n, fontid) or 0xFFFC
+                  next_postcodes[#next_postcodes + 1] = is_char(n, fontid) or obj_repl
                 end
                 table.move(postcodes, saved_after + 1, #postcodes, #next_postcodes + 1, next_postcodes)
 
@@ -573,7 +585,7 @@ function shape(head, firstnode, run)
                 local saved_anchor = saved_anchor - disc_cluster
                 table.move(repcodes, 1, saved_anchor, 1, next_repcodes)
                 for n in traverse(next_rep) do
-                  next_repcodes[#next_repcodes + 1] = is_char(n, fontid) or 0xFFFC
+                  next_repcodes[#next_repcodes + 1] = is_char(n, fontid) or obj_repl
                 end
 
                 local rep = glyphs[disc_glyph].replace.head
@@ -596,7 +608,7 @@ function shape(head, firstnode, run)
                 post = makesub(run, next_postcodes, next_post),
                 cluster = disc_cluster,
                 nextcluster = disc_cluster + 1,
-                codepoint = 0xFFFC,
+                codepoint = obj_repl,
               }
             end
             i = disc_glyph + 1
@@ -637,7 +649,6 @@ function shape(head, firstnode, run)
       ::NEXTCLUSTERFOUND:: -- end
       glyph.nextcluster = nextcluster
 
-      local disc, discindex
       -- Calculate the Unicode code points of this glyph. If cluster did not
       -- change then this is a glyph inside a complex cluster and will be
       -- handled with the start of its cluster.
@@ -651,19 +662,12 @@ function shape(head, firstnode, run)
             -- assert(char == codes[j])
             hex = hex .. to_utf16_hex(char)
             str = str .. utf8.char(char)
-          elseif not discindex and id == disc_t then
-            local props = properties[disc]
-            if not (props and props.zwnj) then
-              disc, discindex = node, j
-            end
           end
           node = getnext(node)
         end
         glyph.tounicode = hex
         glyph.string = str
       end
-      if not fordisc and discindex then
-      end
     end
     return head, firstnode, glyphs, run.len - len
   else
@@ -743,7 +747,6 @@ local function tonodes(head, node, run, glyphs)
   local dir = run.dir
   local fontid = run.font
   local fontdata = font.getfont(fontid)
-  local space = fontdata.parameters.space
   local characters = fontdata.characters
   local hbdata = fontdata.hb
   local hfactor = (fontdata.extend or 1000) / 1000
@@ -758,6 +761,7 @@ local function tonodes(head, node, run, glyphs)
   local lastprops
 
   local scale = hbdata.scale
+  local space = hbdata.space
 
   local haspng = hbshared.haspng
   local fonttype = hbshared.fonttype
@@ -777,9 +781,15 @@ local function tonodes(head, node, run, glyphs)
       nodeindex = glyph.cluster + 1
     elseif nextcluster + 1 == nodeindex then -- Oops, we went too far
       nodeindex = nodeindex - 1
-      local new = inherit(glyph_t, getprev(node), lastprops)
-      setfont(new, fontid)
-      head, node = insertbefore(head, node, new)
+      if node then
+        local new = inherit(glyph_t, 0, getprev(node), lastprops)
+        head, node = insertbefore(head, node, new)
+      else
+        node = tail(head)
+        local new = inherit(glyph_t, 0, node, lastprops)
+        head, node = insertafter(head, node, new)
+      end
+      setfont(node, fontid)
     end
     local gid = glyph.codepoint
     local char = nominals[gid] or gid_offset + gid
@@ -1040,8 +1050,8 @@ function process(head, font, _attr, direction)
   return newhead or head
 end
 
-local function pageliteral(data)
-  local n = newnode(whatsit_t, pdfliteral_t)
+local function pageliteral(data, base)
+  local n = inherit(whatsit_t, pdfliteral_t, base, getproperty(base))
   setwhatsitfield(n, "mode", 1) -- page
   setwhatsitfield(n, "data", data) -- page
   return n
@@ -1059,12 +1069,12 @@ local function post_process(head)
 
     if startactual then
       local actualtext = "/Span<</ActualText<FEFF"..startactual..">>>BDC"
-      head = insertbefore(head, n, pageliteral(actualtext))
+      head = insertbefore(head, n, pageliteral(actualtext, n))
       props[startactual_p] = nil
     end
 
     if endactual then
-      head = insertafter(head, n, pageliteral("EMC"))
+      head = insertafter(head, n, pageliteral("EMC", n))
       props[endactual_p] = nil
     end
 
@@ -1139,6 +1149,8 @@ local function get_glyph_info(n)
   local c = getchar(n)
   if c == 0 then
     return '^^@'
+  elseif c < 0 then -- These are left and right boundaries in ligature lists
+    return ''
   elseif c < 0x110000 then
     return utfchar(c)
   else
